package com.link.core.cllog;

import com.alibaba.fastjson.JSONObject;
import com.google.gson.Gson;
import com.link.base.logs.model.HttpLog;
import com.link.base.user.model.CoreUser;
import com.link.core.basic.interceptor.SecurityInterceptor;
import com.link.core.kafka.KafkaTemplateFacotry;
import com.link.core.kafka.conf.KafkaEnv;
import com.link.core.objlocalhandle.kafka.KafkaEvent;
import com.link.core.objlocalhandle.kafka.ObjLocalKafkaUtils;
import com.link.core.util.CoreYmlConstants;
import com.link.core.util.StringUtils;
import com.link.core.util.UserUtil;
import java.math.BigDecimal;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.text.SimpleDateFormat;
import java.util.Enumeration;
import java.util.Objects;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public class LCLogger {
   private static final Logger logger = LogManager.getLogger(LCLogger.class);
   private static final String SEPARATOR = "^";
   private static final SimpleDateFormat DF = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss Z");
   private static final String IP = getIp();
   private static final String REQ_OUTER_INTERFACE_TOPIC_NAME = "HTTP_LOG";
   private static final Gson gson = new Gson();

   protected static String getIp() {
      String localIp = null;
      String netIp = null;

      Enumeration netInterfaces;
      SocketException ip;
      try {
         netInterfaces = NetworkInterface.getNetworkInterfaces();
      } catch (SocketException var7) {
         ip = var7;
         withException((Throwable)ip);
         return null;
      }

      ip = null;
      boolean finded = false;

      while(netInterfaces.hasMoreElements() && !finded) {
         NetworkInterface ni = (NetworkInterface)netInterfaces.nextElement();
         Enumeration<InetAddress> address = ni.getInetAddresses();

         while(address.hasMoreElements()) {
            InetAddress ip = (InetAddress)address.nextElement();
            if (!ip.isSiteLocalAddress() && !ip.isLoopbackAddress() && ip.getHostAddress().indexOf(":") == -1) {
               netIp = ip.getHostAddress();
               finded = true;
               break;
            }

            if (ip.isSiteLocalAddress() && !ip.isLoopbackAddress() && ip.getHostAddress().indexOf(":") == -1) {
               localIp = ip.getHostAddress();
            }
         }
      }

      return netIp != null && !"".equals(netIp) ? netIp : localIp;
   }

   private static String formatDate(Long currentTimeMillis) {
      synchronized(DF) {
         return DF.format(currentTimeMillis);
      }
   }

   public static LogMessageBuilder info() {
      return new LogMessageBuilder("info", getPreClassName(), Thread.currentThread().getId());
   }

   public static LogMessageBuilder warn() {
      return new LogMessageBuilder("warn", getPreClassName(), Thread.currentThread().getId());
   }

   public static LogMessageBuilder error() {
      return new LogMessageBuilder("error", getPreClassName(), Thread.currentThread().getId());
   }

   private static String getPreClassName() {
      String className = "";
      StackTraceElement[] stackTrace = Thread.currentThread().getStackTrace();
      int length = 2;
      if (stackTrace != null && stackTrace.length > length) {
         className = stackTrace[3].getClassName();
      }

      return className;
   }

   public static void withException(Throwable throwable) {
      if (null != throwable.getStackTrace()) {
         String stack = throwable.toString();
         int len = throwable.getStackTrace().length;
         StringBuilder traces = new StringBuilder();

         for(int i = 0; i < len; ++i) {
            StackTraceElement element = throwable.getStackTrace()[i];
            if (i == 0) {
               traces.append(element);
            } else {
               traces.append("\n").append(element);
            }
         }

         logger.error("\n\t stack: \t" + stack + "\n\t traces: \t" + traces);
      }

   }

   public static void withException(String message) {
      logger.error("\n\t 错误日志: \t" + message);
   }

   private static void sendInfo(String topic, String stack, String traces) {
      KafkaTemplateFacotry.send(topic, exceptionMessageStream(stack, traces));
   }

   private static String exceptionMessageStream(String stack, String traces) {
      String userIdAndName = getUserIdAndName();
      return formatDate(System.currentTimeMillis()) + "^" + IP + "^" + "exception" + "^" + Thread.currentThread().getId() + "^" + LogMessageBuilder.handleNull(getPreClassName()) + "^" + "--" + "^" + "--" + "^" + stack + "^" + traces + "^" + LogMessageBuilder.handleNull(KafkaEnv.getProfile()) + "^" + userIdAndName;
   }

   protected static String getUserIdAndName() {
      String userId = null;
      String userName = null;

      try {
         CoreUser user = UserUtil.getUser();
         if (user != null) {
            userId = user.getId();
            userName = user.getUsername();
         }
      } catch (Exception var3) {
      }

      return userId + "^" + userName;
   }

   public static void sendHttpLog(HttpLog httpLog) {
      CoreUser user = UserUtil.getUser();
      String logJson;
      if (user != null) {
         logJson = user.getId();
         httpLog.setCreatedBy(logJson);
         httpLog.setLastUpdatedBy(logJson);
         httpLog.setCorpid(user.getCorpid());
         httpLog.setPostnId(user.getPostnId());
         httpLog.setOrgId(user.getOrgId());
      }

      try {
         if ("Y".equals(CoreYmlConstants.getLogToES())) {
            Boolean respTimeNull = Objects.isNull(httpLog.getRespTime());
            if (respTimeNull) {
               httpLog.setRespTime(System.currentTimeMillis());
            }

            JSONObject jsonObject = new JSONObject();
            jsonObject.put("requestId", SecurityInterceptor.linkRequestId.get());
            BigDecimal b1 = new BigDecimal(httpLog.getRespTime() - httpLog.getReqTime());
            BigDecimal b2 = new BigDecimal(1000);
            jsonObject.put("takeTime", b1.divide(b2, 3, 4).doubleValue());
            String createdBy = httpLog.getCreatedBy();
            createdBy = StringUtils.isBlank(createdBy) ? "0" : createdBy;
            Long corpid = httpLog.getCorpid();
            corpid = Objects.isNull(corpid) ? 0L : corpid;
            String postnId = httpLog.getPostnId();
            postnId = StringUtils.isBlank(postnId) ? "0" : postnId;
            String orgId = httpLog.getOrgId();
            orgId = StringUtils.isBlank(orgId) ? "0" : orgId;
            String format = "yyyy-MM-dd HH:mm:ss.SSS";
            SimpleDateFormat sdf = new SimpleDateFormat(format);
            String requestTime = sdf.format(httpLog.getReqTime());
            String repostTime = sdf.format(httpLog.getRespTime());
            jsonObject.put("auth", httpLog.getAuth());
            jsonObject.put("respBody", httpLog.getRespBody());
            jsonObject.put("reqParams", httpLog.getReqParams());
            jsonObject.put("url", httpLog.getUrl());
            jsonObject.put("httpStatus", httpLog.getHttpStatus());
            jsonObject.put("errorMsg", httpLog.getErrorMsg());
            jsonObject.put("reqTime", requestTime);
            jsonObject.put("respTime", repostTime);
            jsonObject.put("createdBy", createdBy);
            jsonObject.put("corpid", corpid);
            jsonObject.put("postnId", postnId);
            jsonObject.put("orgId", orgId);
            if (user != null) {
               jsonObject.put("userName", user.getUsername());
            }

            String httpLogJsonStr = JSONObject.toJSONString(jsonObject);
            KafkaEvent kafkaEvent = KafkaEvent.build().topic("HTTP_ES_LOG").event(httpLogJsonStr);
            ObjLocalKafkaUtils.publish(kafkaEvent);
         } else {
            logJson = gson.toJson(httpLog);
            KafkaTemplateFacotry.send("HTTP_LOG", logJson);
         }
      } catch (Exception var16) {
         Exception e = var16;
         logger.error(e.getMessage());
      }

   }
}
